#include "broken.h"
#include <net/socket.h>
#include <net/net_ip.h>

int modem_open( Modem *modem )
{
    int sock = socket( AF_LTE, 0, NPROTO_AT );

	if( sock == -1 )
    {
        DBG( "socket failed, errno %d\n", errno );
        return( -1 );
	}

    DBG( "opened socket, fd %d\n", sock );
    modem->sock = sock;
    memset( modem->buff, 0, sizeof(modem->buff) );
    return( 0 );
}

void modem_close( Modem *modem )
{
    if( modem->sock != -1 )
    {
        close( modem->sock );
        modem->sock = -1;
    }
}

char *modem_readline( Modem *modem, int timeout_ms )
{
    int ret = 0;

    if( modem->pos >= modem->buffsize )
    {
        // need to refill the buffer
        struct timeval tv;
        tv.tv_sec = timeout_ms / 1000;
        tv.tv_usec = (timeout_ms % 1000) * 1000;
        if( setsockopt( modem->sock, SOL_SOCKET, SO_RCVTIMEO, (const char*)&tv, sizeof(tv) ) < 0 )
            DBG( "setsockopt(SO_RECVTIMEO) failed, fd %d, errno %d\n", modem->sock, errno );

        ret = recv( modem->sock, modem->buff, sizeof(modem->buff) - 1, 0 );
        if( ret == -1 )
        {
            if( errno == ETIMEDOUT )
                DBG( "recv[%d] timeout after %dms\n", modem->sock, timeout_ms );
            else
                DBG( "recv[%d] failed, err %d\n", modem->sock, errno );
            return( NULL );
        }
        else
        {
            modem->pos = 0;
            modem->buffsize = ret;
            modem->buff[ret] = 0;
            //DBG( "[fd %d] read: %s\n", modem->sock, modem->buff );
        }
    }

    char *start = modem->buff + modem->pos;

    while( modem->pos < modem->buffsize )
    {
        char *p = modem->buff + modem->pos++;
        if( *p == '\r' || *p == '\n' )
        {
            *p = 0;
            break;
        }
    }

    if( *start )
    {
        DBG( "read[%d]: %s\n", modem->sock, start );
        if( ! strncmp( start, "+CEREG: 5", 9 ) )
            got_modem_connected();
    }
    return( start );
}

int modem_cmd( Modem *modem, const char *cmd, int timeout_ms )
{
    int ret;
    char *line;

    DBG( "send[%d]: %s\n", modem->sock, cmd );
    ret = send( modem->sock, cmd, strlen(cmd), 0 );
    if( ret == -1 )
    {
        DBG( "send[%d] failed, ret %d\n", modem->sock, ret );
        return( -1 );
    }

    while( (line = modem_readline( modem, timeout_ms )) != NULL )
    {
        // finished when we get one of these responses
        if( ! strcmp( line, "OK" ) )
            return( AT_OK );
        else if( ! strcmp( line, "ERROR" ) )
            return( AT_ERROR );
        else if( ! strncmp( line, "+CMS ERROR:", 11 ) )
            return( AT_CMS_ERROR );
        else if( ! strncmp( line, "+CME ERROR:", 11 ) )
            return( AT_CME_ERROR );
    }

    return( -1 );
}

// only a single command needs to be sent; open, send the command and close
int modem_single_cmd( const char *buff, int timeout_ms )
{
    int ret = -1;
    Modem modem = MODEM_INIT;

    if( modem_open( &modem ) == 0 )
    {
        ret = modem_cmd( &modem, buff, timeout_ms );
        modem_close( &modem );
    }

    return( ret );
}
